#include <iostream>
#include <iomanip>
#include <vector>
#include <climits>
using namespace std;

struct Book
{
    string  name;
    string  author;
    int     year;
    string  genre;  // "Biography", "Children", "Computing", "Crime", "Fantasy", "Horror", "Philosophy", "Romance", "Science"
    int     rentedByReader;
};
typedef vector<Book> Books;

struct Reader
{
    string      name;
    string      surname;
    int         cardNumber; 
};
typedef vector<Reader> Readers;

void printBooks             (const Books&     books);
void addBook                (      Books&     books);
void rentBook               (      Books&     books, Readers& readers);
void returnBook             (      Books&     books);
void printReaders           (const Readers&   readers, Books& books);
void addReader              (      Readers&   readers);
void printAvailableBooks    (      Books&     books);
void printLoanedBooks       (      Books&     books);

void printMenu ()
{
    cout << endl
         << "Bibliotekos meniu:" << endl
         << "  1 - spausdinti visu knygu sarasa"     << endl
         << "  2 - spausdinti laisvu knygu sarasa"   << endl
         << "  3 - spausdinti isduotu knygu sarasa" << endl
         << "  4 - prideti nauja knyga"   << endl
         << "  5 - isnuomuoti knyga skaitytojui"   << endl
         << "  6 - knyga grazinama i biblioteka"      << endl
         << endl
         << "  7 - spausdinti skaitytojus"   << endl
         << "  8 - prideti nauja skaitytoja" << endl
         << endl
         << "  0 - baigti darba"   << endl;
}

int main()
{
    Books books;
    Readers readers;
    for (;;)
        {
            cout << "\n----------------------------------------\n";
            cout << "(m - rodyti bibliotekos meniu) > ";
            string key;
            cin >> key;
                 if (key == "0") return 0;
            else if (key == "m") printMenu              ();
            else if (key == "1") printBooks             (books);
            else if (key == "4") addBook                (books);
            else if (key == "5") rentBook               (books, readers);
            else if (key == "6") returnBook             (books);
            else if (key == "7") printReaders           (readers, books);
            else if (key == "8") addReader              (readers);
            else if (key == "2") printAvailableBooks    (books);
            else if (key == "3") printLoanedBooks       (books);
            else cout << endl << "Neteisinga komanda..." << endl;
        }
}

void printBooks(const Books &books)
{
    cout << "-----KNYGOS-----" << endl;
    for (int i = 0; i < books.size(); i++)
    {
        cout << (i+1)
             << ". "    << books[i].name
             << "\t "    << books[i].author
             << "\t "    << books[i].year
             << "\t "    << books[i].genre
             << "\t "    << books[i].rentedByReader
             << endl;
    }

}

void addBook(Books &books)
{
    cin.ignore(INT_MAX, '\n');
    Book book;
    cout << "Iveskite nauja knyga:" << endl;
    cout << "   pavadinimas:    "; getline(cin, book.name);
    cout << "   rasytojas:      "; getline(cin,book.author);
    cout << "   metai:          "; cin >> book.year;
    cout << "   zanras:         "; cin >> book.genre;
    book.rentedByReader = 0;
    cout << "Knyga prideta sekmingai." << endl;
    books.push_back(book);
}

void rentBook(Books &books, Readers &readers)
{
    int bookNo = 0;
    printBooks(books);
    cout << "Iveskit knygos eiles numeri: ";
    cin  >> bookNo;
    if (bookNo <= 0 || bookNo > books.size())
    {
        cout << "KLAIDA: nera knygos su ivestu eiles numeriu.";
        return;
    }
    for (int i = 0; i < readers.size(); i++)
    {
        if (books[bookNo-1].rentedByReader != 0)
        {
            cout << "KLAIDA: knyga jau yra isduota.";
            return;
        }
    }

    int readerNo = 0;
    printReaders(readers, books);
    cout << "Pasirinkite skaitytojo bilieto numeri: ";
    cin  >> readerNo;
    if (readerNo <= 0 || readerNo > readers.size())
    {
        cout << "KLAIDA: nera skaitytojo su ivestu bilieto numeriu.";
        return;
    }
    books[bookNo-1].rentedByReader = readers[readerNo-1].cardNumber;
    cout << "Knyga isduota sekmingai." << endl;
}

void returnBook(Books &books)
{
    int bookNo = 0;
    printBooks(books);
    cout << "Iveskit knygos eiles numeri: ";
    cin  >> bookNo;
    if (bookNo <= 0 || bookNo > books.size())
    {
        cout << "KLAIDA: nera knygos su ivestu eiles numeriu.";
        return;
    }
    books[bookNo-1].rentedByReader = 0;
    cout << "Knyga sekmingai grazinta i biblioteka." << endl;
}

void printReaders(const Readers &readers, Books &books)
{
    cout << "-----SKAITYTOJAI-----" << endl;
    for (int i = 0; i < readers.size(); i++)
    {
        cout << readers[i].cardNumber
             << ". " << readers[i].name
             << " "  << readers[i].surname
             << endl;
    }

    int readerNo = 0;

}

void addReader(Readers &readers)
{
    cin.ignore(INT_MAX, '\n');
    Reader reader;
    cout << "Iveskite nauja skaitytoja: " << endl;
    cout << "   vardas: "; getline(cin,reader.name);
    cout << "   pavarde: "; getline(cin, reader.surname);
    reader.cardNumber = readers.size() +1;
    cout << "Skaitytojas pridetas sekmingai." << endl;
    readers.push_back(reader);
}

void printAvailableBooks(Books &books)
{
    cout << "-----LAISVOS KNYGOS-----" << endl;
    for (int i = 0; i < books.size(); i++)
        if (books[i].rentedByReader == 0)
        {
            cout << (i+1)
                 << ". "    << books[i].name
                 << "\t "    << books[i].author
                 << "\t "    << books[i].year
                 << "\t "    << books[i].genre
                 << "\t "    << books[i].rentedByReader
                 << endl;
        }
}

void printLoanedBooks(Books &books)
{
    cout << "-----ISDUOTOS KNYGOS-----" << endl;
    for (int i = 0; i < books.size(); i++)
        if (books[i].rentedByReader != 0)
        {
            cout << (i+1)
                 << ". "    << books[i].name
                 << "\t "    << books[i].author
                 << "\t "    << books[i].year
                 << "\t "    << books[i].genre
                 << "\t "    << books[i].rentedByReader
                 << endl;
        }
}

